from plaster.tools.utils import utils
from plaster.run.rad_filter import rad_filter
from plaster.run.classify_rf.classify_rf_result import ClassifyRFResult
import numpy as np


def classify_rf(
    classify_rf_params,
    train_rf_result,
    sigproc_result,
    rad_filter_result,
    sim_params,
    progress=None,
):
    if sigproc_result.n_channels != sim_params.n_channels:
        raise ValueError(
            f"Sigproc n_channels ({sigproc_result.n_channels}) does not match "
            f"classifier n_channels ({sim_params.n_channels})"
        )

    if sigproc_result.n_cycles != sim_params.n_cycles:
        raise ValueError(
            f"Sigproc n_cycles ({sigproc_result.n_cycles}) does not match "
            f"classifier n_cycles ({sim_params.n_cycles})"
        )

    # Unlike nn_v2 in which we take care to avoid requiring all of the radmat to be
    # loaded into memory at once, I'm going to apply radmat filtering all at once
    # right here to keep things simple, and because we already know RF doesn't scale
    # and I use it only as a benchmark to compare to nn_v2 performance on scaled-down
    # problem sets.
    radmat = utils.mat_flatter(sigproc_result.sig())
    peak_i = np.arange(radmat.shape[0])

    if rad_filter_result is not None:
        radmat_filter_mask = rad_filter.radmat_filter_mask(rad_filter_result)
        assert radmat_filter_mask.shape[0] == radmat.shape[0]
        radmat = radmat[radmat_filter_mask]
        peak_i = peak_i[radmat_filter_mask]

    pred_pep_iz, scores, all_class_scores = train_rf_result.classifier.classify(
        radmat, progress
    )

    return ClassifyRFResult(
        params=classify_rf_params,
        pred_pep_iz=pred_pep_iz,
        scores=scores,
        all_class_scores=all_class_scores,
        peak_i=peak_i
        # peps_pr=peps_pr,
    )
