"""
    lognormal fit the radiometry intensities coming out of sigprocv2 to produce
    the files and heatmaps generated by pflib ("alex") lognormal_fitter_v2.py
"""

from plaster.tools.pipeline.pipeline import PipelineTask
from plaster.run.sigproc_v1.sigproc_v1_result import SigprocV1Result
from plaster.run.sigproc_v2.sigproc_v2_result import SigprocV2Result
from plaster.run.lnfit.lnfit_params import LNFitParams
from plaster.run.lnfit.lnfit_worker import lnfit


class LNFitTask(PipelineTask):
    def start(self):
        lnfit_params = LNFitParams(**self.config.parameters)

        # We don't need all of sigproc loaded, but do need a couple attrs.  The rest
        # will be loaded on demand.
        load_props = ["n_channels", "n_cycles"]

        try:
            sigproc_result = SigprocV1Result.load_from_folder(
                self.inputs.sigproc_dir, prop_list=load_props
            )
        except FileNotFoundError:
            sigproc_result = SigprocV2Result.load_from_folder(
                self.inputs.sigproc_dir, prop_list=load_props
            )

        lnfit_result = lnfit(lnfit_params, sigproc_result)

        lnfit_result.save()
