"""
Models for illumination.

Each model has a cooresponding C implementation for:
    * classifying. Given a (signal, noise) -> p value
    * generating. Generate random samples from the distribution
"""

from plaster.tools.schema import check


class LogNormalModel:
    def __init__(self, params_per_channel):
        """
        params_per_channel: ndarray (n_channels, 4)
            Where 4 is: (zero_mu, zero_sigma, beta, sigma)
        """
        check.array_t(params_per_channel, ndim=2)
        assert params_per_channel.shape[1] == 4
        self.params_per_channel = params_per_channel


class GaussSvNPerCountModel:
    def __init__(self, params_per_channel_per_count):
        """
        params_per_channel_per_count: ndarray (n_channels, n_count, 5)
            Where 5 is: (mu_signal, mu_noise, sigma_signal, sigma_noise, rho)
        """

        check.array_t(params_per_channel_per_count, ndim=3)
        assert params_per_channel_per_count.shape[2] == 5
        self.params_per_channel_per_count = params_per_channel_per_count


class IllumModel:
    def __init__(self, model):
        check.t(model, (LogNormalModel, GaussSvNPerCountModel))
        self.model = model
