#!/usr/bin/env python

"""
Note: Don't import any module not in the stdlib
"""
import os
import pathlib
import subprocess

from plaster.tools.utils.build_utils import PLASTER_ROOT, any_out_of_date


def build(source_dir, output_dir, force=False):
    cwd = pathlib.Path().absolute()

    c_common_folder = source_dir / "plaster/tools/c_common"
    dst_folder = source_dir / "plaster/run/sigproc_v2/c_gauss2_fitter"
    levmar_folder = dst_folder / "levmar-2.6"

    # It might be more correct to not version control the output of cmake in levmar-2.6 and
    # run cmake during build
    os.chdir(str(levmar_folder))
    make_env = os.environ.copy()
    make_env["ENV_OPTS"] = "-fPIC"
    subprocess.run(["make"], env=make_env, capture_output=True)
    os.chdir(str(cwd))

    c_opts = [
        "-c",
        "-fpic",
        "-O3",
        "-I",
        c_common_folder,
        "-I",
        levmar_folder,
        # "./levmar-2.6",
        "-DDEBUG",
    ]

    def build_c(src_name, include_files):
        base_src_name = pathlib.Path(src_name).stem
        target_o = f"{dst_folder}/_{base_src_name}.o"
        if force or any_out_of_date(
            parents=[src_name, *include_files], children=[target_o],
        ):
            subprocess.run(["gcc", *c_opts, dst_folder / src_name, "-o", target_o])
        return target_o

    common_include_files = [
        f"{c_common_folder}/c_common.h",
    ]
    gauss2_fitter_o = build_c("gauss2_fitter.c", common_include_files)
    c_common_o = build_c(f"{c_common_folder}/c_common.c", common_include_files)

    gauss2_fitter_so = (
        output_dir / "plaster/run/sigproc_v2/c_gauss2_fitter/_gauss2_fitter.so"
    )
    if force or any_out_of_date(
        parents=[gauss2_fitter_o, c_common_o], children=[gauss2_fitter_so]
    ):
        subprocess.run(
            [
                "gcc",
                "-shared",
                "-o",
                str(gauss2_fitter_so),
                gauss2_fitter_o,
                c_common_o,
                "-L",
                "./levmar-2.6",
                "-llevmar",
                "-lm",
                "-llapack",
                "-lblas",
            ]
        )

    assert gauss2_fitter_so.exists()


def build_dev():
    build(source_dir=PLASTER_ROOT, output_dir=PLASTER_ROOT)


if __name__ == "__main__":
    build_dev()
