#!/usr/bin/env python
"""
Note: Don't import any module not in the stdlib
"""

import pathlib
import subprocess

from plaster.tools.utils.build_utils import PLASTER_ROOT, any_out_of_date


def build(source_dir, output_dir, force=False):
    c_common_folder = source_dir / "plaster/tools/c_common"
    dst_folder = source_dir / "plaster/run/sigproc_v2/c_radiometry"

    c_opts = [
        "-c",
        "-fpic",
        "-O3",
        "-I",
        c_common_folder,
        "-I",
        "./csa_spline",
        "-DDEBUG",
    ]

    def build_c(src_name, include_files=tuple()):
        base_src_name = pathlib.Path(src_name).stem
        target_o = f"{dst_folder}/_{base_src_name}.o"
        if force or any_out_of_date(
            parents=[src_name, *include_files], children=[target_o],
        ):
            subprocess.run(["gcc", *c_opts, dst_folder / src_name, "-o", target_o])
        return target_o

    include_files = (
        f"{c_common_folder}/c_common.h",
        f"./csa_spline/csa.h",
    )
    radiometry_o = build_c("radiometry.c", include_files)
    c_common_o = build_c(f"{c_common_folder}/c_common.c", include_files)
    csa_o = build_c("./csa_spline/csa.c")
    svd_o = build_c("./csa_spline/svd.c")

    radiometry_so = output_dir / "plaster/run/sigproc_v2/c_radiometry/_radiometry.so"
    if force or any_out_of_date(
        parents=[radiometry_o, c_common_o], children=[radiometry_so]
    ):
        subprocess.run(
            [
                "gcc",
                "-shared",
                "-o",
                radiometry_so,
                csa_o,
                svd_o,
                radiometry_o,
                c_common_o,
                "-lm",
            ]
        )

    assert radiometry_so.exists()


def build_dev():
    build(source_dir=PLASTER_ROOT, output_dir=PLASTER_ROOT)


if __name__ == "__main__":
    build_dev()
