from plaster.tools.pipeline.pipeline import PipelineTask
from plaster.run.sigproc_v2.sigproc_v2_params import SigprocV2Params
from plaster.run.sigproc_v2 import sigproc_v2_worker as worker
from plaster.run.ims_import.ims_import_result import ImsImportResult
from plaster.run.sigproc_v2 import sigproc_v2_common as common
from plaster.tools.log.log import debug, important, colorful_exception


class SigprocV2Task(PipelineTask):
    def start(self):
        sigproc_v2_params = SigprocV2Params(**self.config.parameters)

        ims_import_result = ImsImportResult.load_from_folder(self.inputs.ims_import)

        if sigproc_v2_params.mode in (common.SIGPROC_V2_ILLUM_CALIB,):
            sigproc_v2_instrument_calib_result = worker.sigproc_instrument_calib(
                sigproc_v2_params, ims_import_result, self.progress
            )

            # Save it to the current output in case it can not be written to the designated path
            calib = sigproc_v2_instrument_calib_result.calib
            calib.save_file("./calib.calib")

        elif sigproc_v2_params.mode == common.SIGPROC_V2_INSTRUMENT_ANALYZE:
            worker.sigproc_analyze(
                sigproc_v2_params, ims_import_result, self.progress, pipeline=self
            )

        else:
            raise ValueError("Unknown sigproc_v2 mode")
