#!/usr/bin/env python
"""
Note: Don't import any module not in the stdlib
"""

import pathlib
import subprocess

from plaster.tools.utils.build_utils import PLASTER_ROOT, any_out_of_date


def build(source_dir, output_dir, force=False):
    c_common_folder = source_dir / "plaster/tools/c_common"
    dst_folder = source_dir / "plaster/run/sim_v2/c"

    c_opts = [
        "-c",
        "-fpic",
        "-O3",
        "-I",
        c_common_folder,
        "-DDEBUG",
    ]

    def build_c(src_name, include_files):
        base_src_name = pathlib.Path(src_name).stem
        target_o = f"{dst_folder}/_{base_src_name}.o"
        if force or any_out_of_date(
            parents=[src_name, *include_files], children=[target_o],
        ):
            subprocess.run(["gcc", *c_opts, src_name, "-o", target_o])
        return target_o

    common_include_files = [
        f"{c_common_folder}/c_common.h",
    ]
    o_filename = build_c(f"{dst_folder}/sim_v2.c", common_include_files)
    c_common_o = build_c(f"{c_common_folder}/c_common.c", common_include_files)

    so_filename = output_dir / "plaster/run/sim_v2/c/_sim_v2.so"
    if force or any_out_of_date(
        parents=[o_filename, c_common_o], children=[so_filename]
    ):
        subprocess.run(
            ["gcc", "-shared", "-o", so_filename, o_filename, c_common_o, "-lm",]
        )

    assert so_filename.exists()


def build_dev():
    build(source_dir=PLASTER_ROOT, output_dir=PLASTER_ROOT)


if __name__ == "__main__":
    build_dev()
