"""
Note: Don't import any module not in the stdlib
"""
import pathlib
import subprocess

from plaster.tools.utils.build_utils import PLASTER_ROOT, any_out_of_date


def build(source_dir, output_dir, force=False):
    # In order to build survey_v2, libflann-dev (debian) or flann-devel (centos/manylinux) must be installed
    flann_include_folder = "/usr/include/flann/"

    c_common_folder = source_dir / "plaster/tools/c_common"
    dst_folder = source_dir / "plaster/run/survey_v2/c"
    c_opts = [
        "-c",
        "-fpic",
        "-O3",
        "-I",
        c_common_folder,
        "-I",
        flann_include_folder,
        "-DDEBUG",
    ]

    def build_c(src_name, include_files):
        base_src_name = pathlib.Path(src_name).stem
        target_o = f"{dst_folder}/_{base_src_name}.o"
        if force or any_out_of_date(
            parents=[src_name, *include_files], children=[target_o],
        ):
            subprocess.run(["gcc", *c_opts, src_name, "-o", target_o])
        return target_o

    common_include_files = [
        f"{c_common_folder}/c_common.h",
    ]
    survey_v2_o = build_c(f"{dst_folder}/survey_v2.c", common_include_files)
    c_common_o = build_c(f"{c_common_folder}/c_common.c", common_include_files)

    survey_v2_so = output_dir / "plaster/run/survey_v2/c/_survey_v2.so"
    if force or any_out_of_date(
        parents=[survey_v2_o, c_common_o], children=[survey_v2_so]
    ):
        subprocess.run(
            [
                "gcc",
                "-shared",
                survey_v2_o,
                c_common_o,
                "-lflann",
                "-llz4",
                "-o",
                survey_v2_so,
            ]
        )

    assert survey_v2_so.exists()


def build_dev():
    build(source_dir=PLASTER_ROOT, output_dir=PLASTER_ROOT)


if __name__ == "__main__":
    build_dev()
