import numpy as np
from zest import zest
from plumbum import local
from plaster.run.sigproc_v2 import synth
from plaster.tools.test_tools.test_tools import (
    integration_before,
    integration_after,
    run_p,
)


@zest.group("integration")
def zest_survey_v2_integration():
    # If test_folder is None it will be set by _before()
    # otherwise you can set it here to reuse a debugging folder
    test_folder = None
    # test_folder = (
    #     "/erisyon/internal/jobs_folder/_integration_tests/it_runs_survey_v2/1613159055"
    # )

    def _before():
        nonlocal test_folder
        if test_folder is None:
            test_folder = integration_before()
        test_folder = local.path(test_folder)

    def _after():
        integration_after()

    def it_runs_survey_v2():
        # Prep for gen
        job_folder = test_folder / "job"

        csv_file = test_folder / "protein.csv"
        csv_contents = """
            Name,UniprotAC,Abundance,POI
            ACE,P12821,28,1
            ACE2,Q9BYF1,12,1
            ACTB,P60709,7,1
        """
        csv_contents = "\n".join(csv_contents.split()) + "\n"
        csv_file.write(csv_contents)

        run_p(
            [
                f"gen",
                f"survey",
                f"--sample=KidneyBiomarkers1",
                f"--job={job_folder}",
                f"--protein_csv={csv_file}",
                f"--label_set=C,M,Y",
                f"--protease=trypsin",
                f"--n_pres=1",
                f"--n_mocks=0",
                f"--n_edmans=15",
                f"--force",
            ]
        )

        run_p(["run", job_folder, "--no_progress"])

    zest()
