import time
from zest import zest
from plumbum import FG, colors, local
from plaster.tools.log.log import h_line


def run_p(cmd):

    if isinstance(cmd, str):
        cmd = [l.strip() for l in cmd.split()]

    with colors.fg.DeepSkyBlue3:
        print()
        print(h_line("- "))
        print(f"ABOUT TO EXECUTE:")
        print("  p " + " ".join(cmd))
        print(h_line("- "))

    p_cmd = local[local.path(local.env["PLASTER_ROOT"]) / "p"]
    rc, so, se = p_cmd.run(cmd)
    print(so)
    print(se)


_test_folder = None


def integration_before():
    """
    Call in the _before of any integration test where a job_folder is needed
    """

    test_name = zest.current_test_name()

    with colors.fg.DeepSkyBlue1:
        print()
        print(h_line("="))
        print(f"STARTING INTEGRATION TEST: '{test_name}'")
        print(f"To re-run only this test, execute:")
        print(f"  p test --integration {test_name}")
        print(h_line("=") + "\n")

    # ZBS switched the following to use jobs_folder to simplify debugging because
    # when a report fails it is easiest to open it in jupyter but
    # jupyter does not have access to files outside of the main tree

    global _test_folder
    assert _test_folder is None
    _test_folder = (
        local.path(local.env["JOBS_FOLDER"])
        / "_integration_tests"
        / test_name
        / str(int(time.time()))
    )
    _test_folder.mkdir()
    return _test_folder


def integration_after():
    """
    Call in the _after of any integration test where a job_folder is needed
    """

    global _test_folder
    assert _test_folder is not None

    test_name = zest.current_test_name()
    error = zest.current_test_error()

    if not error:
        _test_folder.delete()
    else:
        # If the test fails, don't clean up so there's some evidence of the failure
        with colors.fg.red & colors.bold:
            print("\n\n")
            print(h_line("-"))
            print(f"TEST FAILED: '{test_name}'")
            print(
                f"Test test was not cleaned up and is in '{_test_folder}' for inspection."
            )
            print(f"See above blue instructions to re-run.")
            print(h_line("-") + "\n")

    _test_folder = None
